//////////////////////////////////////////////////////
// AnimationApp.cpp
//////////////////////////////////////////////////////

#include <windows.h>
#include <d3d9.h>
#include <d3dx9tex.h>
#include <dxerr9.h>

// Function prototypes.
LRESULT WINAPI WndProc(HWND hWnd, UINT msg,
    WPARAM wParam, LPARAM lParam);
void RegisterWindowClass(HINSTANCE hInstance);
void CreateAppWindow(HINSTANCE hInstance);
WPARAM StartMessageLoop();
HRESULT InitDirect3D();
void Render();
void CleanUpDirect3D();
BOOL IsTimeForNextFrame(); 

// Global variables.
HWND g_hWnd;
IDirect3D9* g_pDirect3D = NULL;
IDirect3DDevice9* g_pDirect3DDevice = NULL;
IDirect3DSurface9* g_pBitmapSurfaces[8];
HRESULT g_hResult = D3D_OK;
char g_szErrorMsg[256];


//////////////////////////////////////////////////////
// WinMain()
//////////////////////////////////////////////////////
INT WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR, INT)
{
    for (int x=0; x<8; ++x)
        g_pBitmapSurfaces[x] = NULL;
    RegisterWindowClass(hInstance);
    CreateAppWindow(hInstance);
    ShowWindow(g_hWnd, SW_SHOWDEFAULT);
    UpdateWindow(g_hWnd);
    HRESULT hResult = InitDirect3D();
    if (SUCCEEDED(hResult))
        WPARAM result = StartMessageLoop();
    CleanUpDirect3D();
    CloseWindow(g_hWnd);
    if (g_hResult != D3D_OK)
        DXTRACE_ERR(g_szErrorMsg, g_hResult);
    return 0;
}

//////////////////////////////////////////////////////
// WndProc()
//////////////////////////////////////////////////////
LRESULT WINAPI WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    switch(msg)
    {
    case WM_CREATE:
        return 0;

    case WM_DESTROY:
        PostQuitMessage( 0 );
        return 0;

    case WM_PAINT:
        ValidateRect(g_hWnd, NULL);
        return 0;

    case WM_KEYDOWN:
        switch(wParam)
        {
        case VK_ESCAPE:
            PostQuitMessage(WM_QUIT);
            break;
        }
    }
    return DefWindowProc(hWnd, msg, wParam, lParam);
}

//////////////////////////////////////////////////////
// RegisterWindowClass()
//////////////////////////////////////////////////////
void RegisterWindowClass(HINSTANCE hInstance)
{
    WNDCLASSEX wc;
    wc.cbSize = sizeof(WNDCLASSEX);
    wc.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
    wc.lpfnWndProc = WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0; 
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = (HCURSOR)LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName = NULL;
    wc.lpszClassName = "AnimationApp";
    wc.hIconSm = NULL;

    RegisterClassEx(&wc);
}

//////////////////////////////////////////////////////
// CreateAppWindow()
//////////////////////////////////////////////////////
void CreateAppWindow(HINSTANCE hInstance)
{
    g_hWnd = CreateWindowEx(
        NULL,
        "AnimationApp",
        "Direct3D Animation Application", 
        WS_OVERLAPPEDWINDOW,
        100,
        100,
        648,
        514,
        GetDesktopWindow(),
        NULL,
        hInstance,
        NULL);
}

//////////////////////////////////////////////////////
// StartMessageLoop()
//////////////////////////////////////////////////////
WPARAM StartMessageLoop()
{
    MSG msg;
    while(1)
    {
        if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
        {
            if (msg.message == WM_QUIT)
                break;
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
        else
        {
            // Use idle time here.
            Render();
        }
    }
    return msg.wParam;
}

//////////////////////////////////////////////////////
// InitDirect3D()
//////////////////////////////////////////////////////
HRESULT InitDirect3D()
{
    g_pDirect3D = Direct3DCreate9(D3D_SDK_VERSION);
    if (g_pDirect3D == NULL)
    {
        MessageBox(g_hWnd,
            "Couldn't create DirectX object.",
            "DirectX Error", MB_OK);
        return E_FAIL;
    }

    HRESULT hResult = g_pDirect3D->CheckDeviceType(D3DADAPTER_DEFAULT, 
        D3DDEVTYPE_HAL, D3DFMT_X8R8G8B8, D3DFMT_X8R8G8B8, FALSE);
    if (hResult != D3D_OK)
    {
        MessageBox(g_hWnd,
            "Sorry. This program won't\nrun on your system.",
            "DirectX Error", MB_OK);
        return E_FAIL;
    }

    D3DPRESENT_PARAMETERS D3DPresentParams;
    ZeroMemory(&D3DPresentParams, sizeof(D3DPRESENT_PARAMETERS));
    D3DPresentParams.Windowed = FALSE;
    D3DPresentParams.BackBufferCount = 1;
    D3DPresentParams.BackBufferWidth = 800;
    D3DPresentParams.BackBufferHeight = 600;
    D3DPresentParams.BackBufferFormat = D3DFMT_X8R8G8B8;
    D3DPresentParams.SwapEffect = D3DSWAPEFFECT_DISCARD;
    D3DPresentParams.hDeviceWindow = g_hWnd;
    hResult = g_pDirect3D->CreateDevice(D3DADAPTER_DEFAULT, 
        D3DDEVTYPE_HAL, g_hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING,
        &D3DPresentParams, &g_pDirect3DDevice);
    if (FAILED(hResult))
    {
        MessageBox(g_hWnd,
            "Failed to create Direct3D device.",
            "DirectX Error", MB_OK);
        return E_FAIL;
    }

	for (int x=0; x<8; ++x)
	{
		g_hResult = g_pDirect3DDevice->CreateOffscreenPlainSurface(400, 400, 
			D3DFMT_X8R8G8B8, D3DPOOL_DEFAULT, &g_pBitmapSurfaces[x], NULL);
		if (FAILED(g_hResult))
		{
			strcpy(g_szErrorMsg, "Error creating bitmap surface.");
			PostQuitMessage(WM_QUIT);
			break;
		}
		char filename[15];
		wsprintf(filename, "%s%d%s", "Frame0", x+1, ".bmp"); 
		g_hResult = D3DXLoadSurfaceFromFile(g_pBitmapSurfaces[x], NULL, NULL,
			filename, NULL, D3DX_DEFAULT, 0, NULL);
		if (FAILED(g_hResult))
		{
			strcpy(g_szErrorMsg, "Couldn't load bitmap file.");
			PostQuitMessage(WM_QUIT);
			break;
		}
}

    return D3D_OK;
}

//////////////////////////////////////////////////////
// CleanUpDirect3D()
//////////////////////////////////////////////////////
void CleanUpDirect3D()
{
	for (int x=0; x<8; ++x)
	{
		if (g_pBitmapSurfaces[x])
			g_pBitmapSurfaces[x]->Release();
	}
    if (g_pDirect3DDevice)
        g_pDirect3DDevice->Release();
    if (g_pDirect3D)
        g_pDirect3D->Release();
}

//////////////////////////////////////////////////////
// Render()
//////////////////////////////////////////////////////
void Render()
{
    if (IsTimeForNextFrame())
    {
        static frameNum = 0;
        frameNum = frameNum + 1;
        if (frameNum == 8)
            frameNum = 0;

        IDirect3DSurface9* pBackBuffer = NULL;
        g_hResult = g_pDirect3DDevice->GetBackBuffer(0, 0,
            D3DBACKBUFFER_TYPE_MONO, &pBackBuffer);
        if (FAILED(g_hResult))
        {
            strcpy(g_szErrorMsg, "Error getting back buffer.");
            PostQuitMessage(WM_QUIT);
            return;
        }

        g_pDirect3DDevice->Clear(0, NULL, D3DCLEAR_TARGET,
            D3DCOLOR_XRGB(0,0,0), 1.0f, 0);

		RECT SrcRect;
		RECT DstRect;
		SrcRect.left = 0;
		SrcRect.right = 399;
		SrcRect.top = 0;
		SrcRect.bottom = 399;
		DstRect.left = 200;
		DstRect.top = 100;
		DstRect.right = 599;
		DstRect.bottom = 499;
	    
		HRESULT hResult = g_pDirect3DDevice->
			StretchRect(g_pBitmapSurfaces[frameNum],
			    &SrcRect, pBackBuffer, &DstRect, D3DTEXF_NONE);

        g_pDirect3DDevice->Present(NULL, NULL, NULL, NULL);
        if (pBackBuffer)
            pBackBuffer->Release();
    }
}

//////////////////////////////////////////////////////
// IsTimeForNextFrame()
//////////////////////////////////////////////////////
BOOL IsTimeForNextFrame()
{
    static DWORD count = 0;
    count = count + 1;
    if (count > 100000)
    {
        count = 0;
        return TRUE;
    }
    else
        return FALSE;
}
