//////////////////////////////////////////////////////
// LightApp.cpp
//////////////////////////////////////////////////////

#include <windows.h>
#include <d3d9.h>
#include <d3dx9.h>

// Function prototypes.
LRESULT WINAPI WndProc(HWND hWnd, UINT msg,
    WPARAM wParam, LPARAM lParam);
void RegisterWindowClass(HINSTANCE hInstance);
void CreateAppWindow(HINSTANCE hInstance);
WPARAM StartMessageLoop();
HRESULT InitDirect3D();
void Render();
void CleanUpDirect3D();
HRESULT InitVertices();
void InitMatrices();
void InitLights();

// Global variables.
HWND g_hWnd;
IDirect3D9* g_pDirect3D = NULL;
IDirect3DDevice9* g_pDirect3DDevice = NULL;
IDirect3DVertexBuffer9* g_pVertexBuf = NULL;
int g_iNumTriangles;

struct CUSTOMVERTEXSTRUCT
{
    float fX,
          fY,
          fZ;
    D3DVECTOR normal;
};

//////////////////////////////////////////////////////
// WinMain()
//////////////////////////////////////////////////////
INT WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR, INT)
{
    RegisterWindowClass(hInstance);
    CreateAppWindow(hInstance);
    ShowWindow(g_hWnd, SW_SHOWDEFAULT);
    UpdateWindow(g_hWnd);
    HRESULT hResult = InitDirect3D();
    if (SUCCEEDED(hResult))
    {
        InitVertices();
        WPARAM result = StartMessageLoop();
    }
    CleanUpDirect3D();
    return 0;
}

//////////////////////////////////////////////////////
// WndProc()
//////////////////////////////////////////////////////
LRESULT WINAPI WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    switch(msg)
    {
    case WM_CREATE:
        return 0;

    case WM_DESTROY:
        PostQuitMessage( 0 );
        return 0;

    case WM_PAINT:
        ValidateRect(g_hWnd, NULL);
        return 0;

    case WM_KEYDOWN:
        switch(wParam)
        {
        case VK_ESCAPE:
            PostQuitMessage(WM_QUIT);
            break;
        }
    }
    return DefWindowProc(hWnd, msg, wParam, lParam);
}

//////////////////////////////////////////////////////
// RegisterWindowClass()
//////////////////////////////////////////////////////
void RegisterWindowClass(HINSTANCE hInstance)
{
    WNDCLASSEX wc;
    wc.cbSize = sizeof(WNDCLASSEX);
    wc.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
    wc.lpfnWndProc = WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0; 
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = (HCURSOR)LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName = NULL;
    wc.lpszClassName = "LightApp";
    wc.hIconSm = NULL;

    RegisterClassEx(&wc);
}

//////////////////////////////////////////////////////
// CreateAppWindow()
//////////////////////////////////////////////////////
void CreateAppWindow(HINSTANCE hInstance)
{
    g_hWnd = CreateWindowEx(
        NULL,
        "LightApp",
        "Direct3D Lighting Application", 
        WS_OVERLAPPEDWINDOW,
        100,
        100,
        648,
        514,
        GetDesktopWindow(),
        NULL,
        hInstance,
        NULL);
}

//////////////////////////////////////////////////////
// StartMessageLoop()
//////////////////////////////////////////////////////
WPARAM StartMessageLoop()
{
    MSG msg;
    while(1)
    {
        if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
        {
            if (msg.message == WM_QUIT)
                break;
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
        else
        {
            // Use idle time here.
            Render();
        }
    }
    return msg.wParam;
}

//////////////////////////////////////////////////////
// InitDirect3D()
//////////////////////////////////////////////////////
HRESULT InitDirect3D()
{
    if((g_pDirect3D = Direct3DCreate9(D3D_SDK_VERSION)) == NULL)
        return E_FAIL;

    D3DPRESENT_PARAMETERS D3DPresentParams;
    ZeroMemory(&D3DPresentParams, sizeof(D3DPRESENT_PARAMETERS));
    D3DPresentParams.Windowed = TRUE;
    D3DPresentParams.BackBufferFormat = D3DFMT_UNKNOWN;
    D3DPresentParams.SwapEffect = D3DSWAPEFFECT_DISCARD;
    D3DPresentParams.hDeviceWindow = g_hWnd;
    D3DPresentParams.EnableAutoDepthStencil = TRUE;
    D3DPresentParams.AutoDepthStencilFormat = D3DFMT_D16;

    if (FAILED(g_pDirect3D->CreateDevice(D3DADAPTER_DEFAULT, 
            D3DDEVTYPE_HAL, g_hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING,
            &D3DPresentParams, &g_pDirect3DDevice)))
        return E_FAIL;

    // Turn off culling, and turn on Z buffering.
    g_pDirect3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
    g_pDirect3DDevice->SetRenderState(D3DRS_ZENABLE, TRUE);

    return D3D_OK;
}

//////////////////////////////////////////////////////
// CleanUpDirect3D()
//////////////////////////////////////////////////////
void CleanUpDirect3D()
{
    if (g_pVertexBuf)
    {
        g_pVertexBuf->Release(); 
        g_pVertexBuf = NULL;
    }
    if (g_pDirect3DDevice)
    {
        g_pDirect3DDevice->Release();
        g_pDirect3DDevice = NULL;
    }
    if (g_pDirect3D)
    {
        g_pDirect3D->Release();
        g_pDirect3D = NULL;
    }
}

//////////////////////////////////////////////////////
// Render()
//////////////////////////////////////////////////////
VOID Render()
{
    g_pDirect3DDevice->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,
        D3DCOLOR_XRGB(0,0,0), 1.0f, 0);

    if( SUCCEEDED(g_pDirect3DDevice->BeginScene()))
    {
        InitLights();
        InitMatrices();
        g_pDirect3DDevice->SetStreamSource(0, g_pVertexBuf, 0, 
            sizeof(CUSTOMVERTEXSTRUCT));
        g_pDirect3DDevice->SetFVF(D3DFVF_XYZ|D3DFVF_NORMAL);
        g_pDirect3DDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 
            0, g_iNumTriangles);
        g_pDirect3DDevice->EndScene();
    }

    g_pDirect3DDevice->Present( NULL, NULL, NULL, NULL );
}

//////////////////////////////////////////////////////
// InitVertices()
//////////////////////////////////////////////////////
HRESULT InitVertices()
{
    CUSTOMVERTEXSTRUCT cube[] =
    {
        // Front of cube.
        {-0.5f,  0.5f, -0.5f, 0.0f, 0.0f, 1.0f}, // Top left.
        { 0.5f, -0.5f, -0.5f, 0.0f, 0.0f, 1.0f}, // Bottom right.
        {-0.5f, -0.5f, -0.5f, 0.0f, 0.0f, 1.0f}, // Bottom left.
        {-0.5f,  0.5f, -0.5f, 0.0f, 0.0f, 1.0f}, // Top left.
        { 0.5f,  0.5f, -0.5f, 0.0f, 0.0f, 1.0f}, // Top right.
        { 0.5f, -0.5f, -0.5f, 0.0f, 0.0f, 1.0f},  // Bottom right.

        // Right side of cube.
        {0.5f,  0.5f, -0.5f, 1.0f, 0.0f, 0.0f},
        {0.5f, -0.5f,  0.5f, 1.0f, 0.0f, 0.0f},
        {0.5f, -0.5f, -0.5f, 1.0f, 0.0f, 0.0f},
        {0.5f,  0.5f, -0.5f, 1.0f, 0.0f, 0.0f},
        {0.5f,  0.5f,  0.5f, 1.0f, 0.0f, 0.0f},
        {0.5f, -0.5f,  0.5f, 1.0f, 0.0f, 0.0f},

        // Back of cube.
        { 0.5f,  0.5f, 0.5f, 0.0f, 0.0f, 1.0f},
        {-0.5f, -0.5f, 0.5f, 0.0f, 0.0f, 1.0f},
        { 0.5f, -0.5f, 0.5f, 0.0f, 0.0f, 1.0f},
        { 0.5f,  0.5f, 0.5f, 0.0f, 0.0f, 1.0f},
        {-0.5f,  0.5f, 0.5f, 0.0f, 0.0f, 1.0f},
        {-0.5f, -0.5f, 0.5f, 0.0f, 0.0f, 1.0f},

        // Left side of cube.
        {-0.5f,  0.5f,  0.5f, -1.0f, 0.0f, 0.0f},
        {-0.5f, -0.5f, -0.5f, -1.0f, 0.0f, 0.0f},
        {-0.5f, -0.5f,  0.5f, -1.0f, 0.0f, 0.0f},
        {-0.5f,  0.5f,  0.5f, -1.0f, 0.0f, 0.0f},
        {-0.5f,  0.5f, -0.5f, -1.0f, 0.0f, 0.0f},
        {-0.5f, -0.5f, -0.5f, -1.0f, 0.0f, 0.0f},

        // Top of cube.
        {-0.5f, 0.5f, -0.5f, 0.0f, 1.0f, 0.0f},
        {-0.5f, 0.5f,  0.5f, 0.0f, 1.0f, 0.0f},
        { 0.5f, 0.5f, -0.5f, 0.0f, 1.0f, 0.0f},
        {-0.5f, 0.5f,  0.5f, 0.0f, 1.0f, 0.0f},
        { 0.5f, 0.5f,  0.5f, 0.0f, 1.0f, 0.0f},
        { 0.5f, 0.5f, -0.5f, 0.0f, 1.0f, 0.0f},

        // Bottom of cube.
        {-0.5f, -0.5f, -0.5f, 0.0f, -1.0f, 0.0f},
        { 0.5f, -0.5f, -0.5f, 0.0f, -1.0f, 0.0f},
        {-0.5f, -0.5f,  0.5f, 0.0f, -1.0f, 0.0f},
        { 0.5f, -0.5f, -0.5f, 0.0f, -1.0f, 0.0f},
        { 0.5f, -0.5f,  0.5f, 0.0f, -1.0f, 0.0f},
        {-0.5f, -0.5f,  0.5f, 0.0f, -1.0f, 0.0f}
    };

    // Create the vertex buffer.
    if(FAILED(g_pDirect3DDevice ->CreateVertexBuffer(sizeof(cube),
            0, D3DFVF_XYZ|D3DFVF_NORMAL, D3DPOOL_DEFAULT, 
            &g_pVertexBuf, NULL)))
        return E_FAIL;

    CUSTOMVERTEXSTRUCT* pVertices;
    if(FAILED(g_pVertexBuf->Lock( 0, 0, (void**)&pVertices, 0)))
        return E_FAIL;
    memcpy(pVertices, cube, sizeof(cube));
    g_pVertexBuf->Unlock();

    g_iNumTriangles = (sizeof(cube) / sizeof(float)) / 18;

    return S_OK;
}

//////////////////////////////////////////////////////
// InitMatrices()
//////////////////////////////////////////////////////
VOID InitMatrices()
{
    D3DXMATRIXA16 matWorld;
    D3DXMatrixIdentity(&matWorld);
    static float angle = 0.0f;
    D3DXMatrixRotationY(&matWorld, angle);
    angle += 0.01f;
    g_pDirect3DDevice->SetTransform(D3DTS_WORLD, &matWorld);

    D3DXVECTOR3 vEyePt(0.0f, 1.0f, -3.0f);
    D3DXVECTOR3 vLookatPt(0.0f, 0.0f, 0.0f);
    D3DXVECTOR3 vUpVec(0.0f, 1.0f, 0.0f);
    D3DXMATRIXA16 matView;
    D3DXMatrixLookAtLH(&matView, &vEyePt, &vLookatPt, &vUpVec);
    g_pDirect3DDevice->SetTransform(D3DTS_VIEW, &matView);

    D3DXMATRIXA16 matProj;
    D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI/4, 1.0f, 1.0f, 100.0f);
    g_pDirect3DDevice->SetTransform(D3DTS_PROJECTION, &matProj);
}

//////////////////////////////////////////////////////
// InitLights()
//////////////////////////////////////////////////////
VOID InitLights()
{
    D3DMATERIAL9 material;
    ZeroMemory(&material, sizeof(material));
    material.Ambient.r = 0.5f;
    material.Ambient.g = 0.75f;
    material.Ambient.b = 0.75f;
    material.Ambient.a = 0.0f;
    g_pDirect3DDevice->SetMaterial(&material);
    g_pDirect3DDevice-> SetRenderState(D3DRS_AMBIENT,
        D3DCOLOR_ARGB(0, 255, 255, 255));
}
