//////////////////////////////////////////////////////
// VertexAlphaApp.cpp
//////////////////////////////////////////////////////

#include <windows.h>
#include <d3d9.h>
#include <d3dx9.h>

// Function prototypes.
LRESULT WINAPI WndProc(HWND hWnd, UINT msg,
    WPARAM wParam, LPARAM lParam);
void RegisterWindowClass(HINSTANCE hInstance);
void CreateAppWindow(HINSTANCE hInstance);
WPARAM StartMessageLoop();
HRESULT InitDirect3D();
void Render();
void CleanUpDirect3D();
HRESULT InitVertices();
void InitMatrices();
void InitLights();

// Global variables.
HWND g_hWnd;
IDirect3D9* g_pDirect3D = NULL;
IDirect3DDevice9* g_pDirect3DDevice = NULL;
IDirect3DVertexBuffer9* g_pSolidSquareVertexBuf = NULL;
IDirect3DVertexBuffer9* g_pTransparentSquareVertexBuf = NULL;
IDirect3DTexture9* g_pSolidSquareTexture;
IDirect3DTexture9* g_pTransparentSquareTexture;

struct CUSTOMVERTEXSTRUCT
{
    D3DVECTOR coords;
    D3DVECTOR normal;
    DWORD dwColor;
    float tu, tv;
};

//////////////////////////////////////////////////////
// WinMain()
//////////////////////////////////////////////////////
INT WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR, INT)
{
    RegisterWindowClass(hInstance);
    CreateAppWindow(hInstance);
    ShowWindow(g_hWnd, SW_SHOWDEFAULT);
    UpdateWindow(g_hWnd);
    HRESULT hResult = InitDirect3D();
    if (SUCCEEDED(hResult))
    {
        InitVertices();
        WPARAM result = StartMessageLoop();
    }
    CleanUpDirect3D();
    return 0;
}

//////////////////////////////////////////////////////
// WndProc()
//////////////////////////////////////////////////////
LRESULT WINAPI WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    switch(msg)
    {
    case WM_CREATE:
        return 0;

    case WM_DESTROY:
        PostQuitMessage( 0 );
        return 0;

    case WM_PAINT:
        ValidateRect(g_hWnd, NULL);
        return 0;

    case WM_KEYDOWN:
        switch(wParam)
        {
        case VK_ESCAPE:
            PostQuitMessage(WM_QUIT);
            break;
        }
    }
    return DefWindowProc(hWnd, msg, wParam, lParam);
}

//////////////////////////////////////////////////////
// RegisterWindowClass()
//////////////////////////////////////////////////////
void RegisterWindowClass(HINSTANCE hInstance)
{
    WNDCLASSEX wc;
    wc.cbSize = sizeof(WNDCLASSEX);
    wc.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
    wc.lpfnWndProc = WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0; 
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = (HCURSOR)LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName = NULL;
    wc.lpszClassName = "VertexAlphaApp";
    wc.hIconSm = NULL;

    RegisterClassEx(&wc);
}

//////////////////////////////////////////////////////
// CreateAppWindow()
//////////////////////////////////////////////////////
void CreateAppWindow(HINSTANCE hInstance)
{
    g_hWnd = CreateWindowEx(
        NULL,
        "VertexAlphaApp",
        "Direct3D Vertex Alpha Application", 
        WS_OVERLAPPEDWINDOW,
        100,
        100,
        648,
        514,
        GetDesktopWindow(),
        NULL,
        hInstance,
        NULL);
}

//////////////////////////////////////////////////////
// StartMessageLoop()
//////////////////////////////////////////////////////
WPARAM StartMessageLoop()
{
    MSG msg;
    while(1)
    {
        if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
        {
            if (msg.message == WM_QUIT)
                break;
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
        else
        {
            // Use idle time here.
            Render();
        }
    }
    return msg.wParam;
}

//////////////////////////////////////////////////////
// InitDirect3D()
//////////////////////////////////////////////////////
HRESULT InitDirect3D()
{
    if((g_pDirect3D = Direct3DCreate9(D3D_SDK_VERSION)) == NULL)
        return E_FAIL;

    D3DPRESENT_PARAMETERS D3DPresentParams;
    ZeroMemory(&D3DPresentParams, sizeof(D3DPRESENT_PARAMETERS));
    D3DPresentParams.Windowed = TRUE;
    D3DPresentParams.BackBufferFormat = D3DFMT_UNKNOWN;
    D3DPresentParams.SwapEffect = D3DSWAPEFFECT_DISCARD;
    D3DPresentParams.hDeviceWindow = g_hWnd;
    D3DPresentParams.EnableAutoDepthStencil = TRUE;
    D3DPresentParams.AutoDepthStencilFormat = D3DFMT_D16;

    if (FAILED(g_pDirect3D->CreateDevice(D3DADAPTER_DEFAULT, 
            D3DDEVTYPE_HAL, g_hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING,
            &D3DPresentParams, &g_pDirect3DDevice)))
        return E_FAIL;

    g_pDirect3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
    g_pDirect3DDevice->SetRenderState(D3DRS_ZENABLE, TRUE);
    g_pDirect3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, true);
    g_pDirect3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
    g_pDirect3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

    HRESULT hResult = D3DXCreateTextureFromFile(g_pDirect3DDevice, 
        "texture03.bmp", &g_pTransparentSquareTexture);
    hResult = D3DXCreateTextureFromFile(g_pDirect3DDevice, 
        "texture02.bmp", &g_pSolidSquareTexture);
    g_pDirect3DDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_DIFFUSE);

    return D3D_OK;
}

//////////////////////////////////////////////////////
// CleanUpDirect3D()
//////////////////////////////////////////////////////
void CleanUpDirect3D()
{
    if (g_pSolidSquareVertexBuf)
    {
        g_pSolidSquareVertexBuf->Release(); 
        g_pSolidSquareVertexBuf = NULL;
    }
    if (g_pTransparentSquareVertexBuf)
    {
        g_pTransparentSquareVertexBuf->Release(); 
        g_pTransparentSquareVertexBuf = NULL;
    }
    if (g_pSolidSquareTexture)
    {
        g_pSolidSquareTexture->Release();
        g_pSolidSquareTexture = NULL;
    }
    if (g_pTransparentSquareTexture)
    {
        g_pTransparentSquareTexture->Release();
        g_pTransparentSquareTexture = NULL;
    }
    if (g_pDirect3DDevice)
    {
        g_pDirect3DDevice->Release();
        g_pDirect3DDevice = NULL;
    }
    if (g_pDirect3D)
    {
        g_pDirect3D->Release();
        g_pDirect3D = NULL;
    }
}

//////////////////////////////////////////////////////
// Render()
//////////////////////////////////////////////////////
VOID Render()
{
    g_pDirect3DDevice->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,
        D3DCOLOR_XRGB(0,0,0), 1.0f, 0);

    if( SUCCEEDED(g_pDirect3DDevice->BeginScene()))
    {
        InitLights();
        InitMatrices();
        g_pDirect3DDevice->SetFVF(D3DFVF_XYZ|D3DFVF_NORMAL | 
            D3DFVF_DIFFUSE | D3DFVF_TEX1);
        g_pDirect3DDevice->SetTexture(0, g_pSolidSquareTexture);
        g_pDirect3DDevice->SetStreamSource(0, g_pSolidSquareVertexBuf, 0, 
            sizeof(CUSTOMVERTEXSTRUCT));
        g_pDirect3DDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 
            0, 2);
        g_pDirect3DDevice->SetTexture(0, g_pTransparentSquareTexture);
        g_pDirect3DDevice->SetStreamSource(0, g_pTransparentSquareVertexBuf, 0,
            sizeof(CUSTOMVERTEXSTRUCT));
        g_pDirect3DDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 
            0, 2);
        g_pDirect3DDevice->EndScene();
    }

    g_pDirect3DDevice->Present( NULL, NULL, NULL, NULL );
}

//////////////////////////////////////////////////////
// InitVertices()
//////////////////////////////////////////////////////
HRESULT InitVertices()
{
    CUSTOMVERTEXSTRUCT solidSquare[] =
    {
        {-1.0f,  1.0f, 1.0f, 0.0f, 0.0f, -1.0f, 
            D3DCOLOR_ARGB(255,255,255,255), 0.0f, 0.0f},
        { 1.0f, -1.0f, 1.0f, 0.0f, 0.0f, -1.0f, 
            D3DCOLOR_ARGB(255,255,255,255), 1.0f, 1.0f},
        {-1.0f, -1.0f, 1.0f, 0.0f, 0.0f, -1.0f, 
            D3DCOLOR_ARGB(255,255,255,255), 0.0f, 1.0f},
        {-1.0f,  1.0f, 1.0f, 0.0f, 0.0f, -1.0f,
            D3DCOLOR_ARGB(255,255,255,255), 0.0f, 0.0f},
        { 1.0f,  1.0f, 1.0f, 0.0f, 0.0f, -1.0f,
            D3DCOLOR_ARGB(255,255,255,255), 1.0f, 0.0f},
        { 1.0f, -1.0f, 1.0f, 0.0f, 0.0f, -1.0f,
            D3DCOLOR_ARGB(255,255,255,255), 1.0f, 1.0f}
    };

    CUSTOMVERTEXSTRUCT transparentSquare[] =
    {
        { 0.75f,  0.75f, 0.2f, 0.0f, 0.0f, 1.0f, 
            D3DCOLOR_ARGB(170,255,255,255), 0.0f, 0.0f},
        {-0.75f, -0.75f, 0.2f, 0.0f, 0.0f, 1.0f, 
            D3DCOLOR_ARGB(170,255,255,255), 1.0f, 1.0f},
        { 0.75f, -0.75f, 0.2f, 0.0f, 0.0f, 1.0f, 
            D3DCOLOR_ARGB(170,255,255,255), 0.0f, 1.0f},
        { 0.75f,  0.75f, 0.2f, 0.0f, 0.0f, 1.0f, 
            D3DCOLOR_ARGB(170,255,255,255), 0.0f, 0.0f},
        {-0.75f,  0.75f, 0.2f, 0.0f, 0.0f, 1.0f, 
            D3DCOLOR_ARGB(170,255,255,255), 1.0f, 0.0f},
        {-0.75f, -0.75f, 0.2f, 0.0f, 0.0f, 1.0f, 
            D3DCOLOR_ARGB(170,255,255,255), 1.0f, 1.0f}
    };

    if(FAILED(g_pDirect3DDevice ->CreateVertexBuffer(sizeof(solidSquare),
            0, D3DFVF_XYZ|D3DFVF_NORMAL | D3DFVF_DIFFUSE | D3DFVF_TEX1,
            D3DPOOL_DEFAULT, &g_pSolidSquareVertexBuf, NULL)))
        return E_FAIL;
    if(FAILED(g_pDirect3DDevice ->CreateVertexBuffer(sizeof(transparentSquare),
            0, D3DFVF_XYZ|D3DFVF_NORMAL | D3DFVF_DIFFUSE | D3DFVF_TEX1, 
            D3DPOOL_DEFAULT, &g_pTransparentSquareVertexBuf, NULL)))
        return E_FAIL;

    CUSTOMVERTEXSTRUCT* pVertices;
    if(FAILED(g_pSolidSquareVertexBuf->Lock(0, 0, (void**)&pVertices, 0)))
        return E_FAIL;
    memcpy(pVertices, solidSquare, sizeof(solidSquare));
    g_pSolidSquareVertexBuf->Unlock();
    if(FAILED(g_pTransparentSquareVertexBuf->Lock(0, 0, (void**)&pVertices, 0)))
        return E_FAIL;
    memcpy(pVertices, transparentSquare, sizeof(transparentSquare));
    g_pTransparentSquareVertexBuf->Unlock();

    return S_OK;
}

//////////////////////////////////////////////////////
// InitMatrices()
//////////////////////////////////////////////////////
VOID InitMatrices()
{
    D3DXMATRIXA16 matWorld;
    D3DXMatrixIdentity(&matWorld);
    static float angle = 0.0f;
    D3DXMatrixRotationY(&matWorld, angle);
    angle += 0.01f;
    g_pDirect3DDevice->SetTransform(D3DTS_WORLD, &matWorld);

    D3DXVECTOR3 vEyePt(0.0f, 1.0f, -4.0f);
    D3DXVECTOR3 vLookatPt(0.0f, 0.0f, 0.0f);
    D3DXVECTOR3 vUpVec(0.0f, 1.0f, 0.0f);
    D3DXMATRIXA16 matView;
    D3DXMatrixLookAtLH(&matView, &vEyePt, &vLookatPt, &vUpVec);
    g_pDirect3DDevice->SetTransform(D3DTS_VIEW, &matView);

    D3DXMATRIXA16 matProj;
    D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI/4, 1.0f, 1.0f, 100.0f);
    g_pDirect3DDevice->SetTransform(D3DTS_PROJECTION, &matProj);
}

//////////////////////////////////////////////////////
// InitLights()
//////////////////////////////////////////////////////
VOID InitLights()
{
    D3DLIGHT9 light0;
    ZeroMemory(&light0, sizeof(light0));

    light0.Type = D3DLIGHT_SPOT;
    light0.Diffuse.r  = 1.0f;
    light0.Diffuse.g  = 1.0f;
    light0.Diffuse.b  = 1.0f;
    light0.Ambient.r  = 1.0f;
    light0.Ambient.g  = 1.0f;
    light0.Ambient.b  = 1.0f;
    light0.Specular.r  = 1.0f;
    light0.Specular.g  = 1.0f;
    light0.Specular.b  = 1.0f;
    light0.Position.x = 2.0f;
    light0.Position.y = 2.0f;
    light0.Position.z = 0.0f;
    light0.Direction.x = -1.0f;
    light0.Direction.y = -1.0f;
    light0.Direction.z = 0.0f;
    light0.Attenuation0 = 1.0f;
    light0.Range = 50.0f;
    light0.Phi = 2.0f;
    light0.Theta = 1.0f;
    light0.Falloff = 1.0f;

    g_pDirect3DDevice->SetLight(0, &light0);
    g_pDirect3DDevice->LightEnable(0, TRUE);

	D3DMATERIAL9 material;
	ZeroMemory(&material, sizeof(material));
	material.Diffuse.r = 1.0f;
	material.Diffuse.g = 1.0f;
	material.Diffuse.b = 1.0f;
	material.Diffuse.a = 1.0f;
	material.Ambient.r = 1.0f;
	material.Ambient.g = 1.0f;
	material.Ambient.b = 1.0f;
	material.Ambient.a = 1.0f;
	g_pDirect3DDevice->SetMaterial(&material);
}
